{% extends 'defaults/base.html' %} {% block template_info_title %}
<h1><span class="temp_type">ARTICLE</span> DEFAULT TEMPLATE</h1>
{% endblock %}
{% block content %}
<h2>{{page.title}}</h2>
<div>{{page.content|safe}}</div>
{% endblock %}
