{% extends "admin/base.html" %}
{% load static %}

{% block extrahead %}
<link rel="icon" href="{% static "admin/img/favicon.ico" %}" sizes="48x48" />
<link rel="stylesheet" type="text/css" href="{% static "admin/css/responsive.css" %}" />
{% endblock %}